﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Media;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Shapes;

namespace Bezier3
{
  public partial class MainWindow : Window
  {
    const int cRad = 8;
    PathFigure f;
    BezierSegment bs;

    public MainWindow()
    {
      InitializeComponent();

      PathGeometry geo = (PathGeometry)bezier.Data;

      f = geo.Figures[0];
      DefinePoint("e0", f.StartPoint);

      bs = (BezierSegment)f.Segments[0];
      DefinePoint("e1", bs.Point1);
      DefinePoint("e2", bs.Point2);
      DefinePoint("e3", bs.Point3);
    }

    private void DefinePoint(string strName, Point pt)
    {
      Ellipse ell = new Ellipse();
      ell.Name = strName;
      ell.Width = cRad * 2;
      ell.Height = cRad * 2;
      ell.Stroke = Brushes.Black;
      ell.Fill = Brushes.Red;
      ell.SetValue(Canvas.LeftProperty, pt.X - cRad);
      ell.SetValue(Canvas.TopProperty, pt.Y - cRad);
      ell.MouseLeftButtonDown += new MouseButtonEventHandler(MouseLeftButtonDown);
      ell.MouseLeftButtonUp += new MouseButtonEventHandler(MouseLeftButtonUp);
      ell.MouseMove += new MouseEventHandler(MouseMove);
      cont.Children.Add(ell);
    }

    private new void MouseLeftButtonDown(object sender, MouseEventArgs e)
    {
      Ellipse ell = sender as Ellipse;
      Point pt = e.GetPosition(cont);
      ell.CaptureMouse();
      SetNewPoint(ell, pt);
    }

    private new void MouseLeftButtonUp(object sender, MouseEventArgs e)
    {
      Ellipse ell = sender as Ellipse;
      Point pt = e.GetPosition(cont);

      if ((pt.X < 0) || (pt.Y < 0) || (pt.X > cont.ActualWidth) || (pt.Y > cont.ActualHeight))
      {
        pt = new Point(cont.ActualWidth / 2, cont.ActualHeight / 2);
      }

      ell.ReleaseMouseCapture();
      SetNewPoint(ell, pt);
    }

    private new void MouseMove(object sender, MouseEventArgs e)
    {
      if (e.LeftButton == MouseButtonState.Pressed)
      {
        Ellipse ell = sender as Ellipse;
        Point pt = e.GetPosition(cont);
        SetNewPoint(ell, pt);
      }
    }

    private void SetNewPoint(Ellipse ell, Point pt)
    {
      ell.SetValue(Canvas.LeftProperty, pt.X - cRad);
      ell.SetValue(Canvas.TopProperty, pt.Y - cRad);

      if (ell.Name == "e0")
      {
        f.StartPoint = pt;
      }
      else if (ell.Name == "e1")
      {
        bs.Point1 = pt;
      }
      else if (ell.Name == "e2")
      {
        bs.Point2 = pt;
      }
      else if (ell.Name == "e3")
      {
        bs.Point3 = pt;
      }
    }
  }
}
