﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Media.Media3D;

namespace Button3D
{
  public partial class MainWindow : Window
  {
    private int iRinge = 16;
    private int iSeg = 16;

    public MainWindow()
    {
      InitializeComponent();

      ShowKugel();
    }

    private void OnStartExit(object sender, EventArgs e)
    {
      this.Close();
    }

    private void ShowKugel()
    {
      double dRadius = 5.0;

      // Kamera definieren
      PerspectiveCamera camera = new PerspectiveCamera();
      camera.FarPlaneDistance = 100;
      camera.NearPlaneDistance = 1;
      camera.LookDirection = new Vector3D(-10, -10, -7);
      camera.UpDirection = new Vector3D(0, 1, 0);
      camera.Position = new Point3D(14, 13, 10);
      camera.FieldOfView = 70;
      this.view.Camera = camera;

      // Licht definieren
      ModelVisual3D lightModel = new ModelVisual3D();
      Vector3D lightVector = new Vector3D(-20, -30, -10);
      DirectionalLight light = new DirectionalLight(Colors.White, lightVector);
      lightModel.Content = light;

      // Darstellung in Viewport3D
      ModelVisual3D model = new ModelVisual3D();
      model.Content = ModelKugel(iRinge, iSeg, dRadius);
      this.view.Children.Add(lightModel);
      this.view.Children.Add(model);
    }

    private Model3DGroup ModelKugel(int iRinge, int iSeg, double dSize)
    {
      // Berechnung der Kugeloberfläche in Segmenten und Ringen

      Model3DGroup kugel = new Model3DGroup();

      double dRingWinkel = Math.PI / iRinge;
      double dSegWinkel = 2.0 * Math.PI / iSeg;

      for (int iR = 0; iR < iRinge; iR++)
      {
        double r0 = Math.Sin(iR * dRingWinkel);
        double r1 = Math.Sin((iR + 1) * dRingWinkel);
        double y0 = Math.Cos(iR * dRingWinkel);
        double y1 = Math.Cos((iR + 1) * dRingWinkel);

        for (int iS = 0; iS <= iSeg; iS++)
        {
          double x0 = r0 * Math.Sin(iS * dSegWinkel);
          double z0 = r0 * Math.Cos(iS * dSegWinkel);
          double x1 = r1 * Math.Sin(iS * dSegWinkel);
          double z1 = r1 * Math.Cos(iS * dSegWinkel);

          Point3D pt1 = new Point3D(x0 * dSize, y0 * dSize, z0 * dSize);
          Point3D pt2 = new Point3D(x1 * dSize, y1 * dSize, z1 * dSize);

          x0 = r0 * Math.Sin((iS + 1) * dSegWinkel);
          z0 = r0 * Math.Cos((iS + 1) * dSegWinkel);
          x1 = r1 * Math.Sin((iS + 1) * dSegWinkel);
          z1 = r1 * Math.Cos((iS + 1) * dSegWinkel);

          Point3D pt3 = new Point3D(x0 * dSize, y0 * dSize, z0 * dSize);
          Point3D pt4 = new Point3D(x1 * dSize, y1 * dSize, z1 * dSize);

          kugel.Children.Add(TriangleModel(pt1, pt2, pt4, pt3));
        }
      }

      return kugel;
    }

    private Model3DGroup TriangleModel(Point3D pt0, Point3D pt1, Point3D pt2, Point3D pt3)
    {
      // Ein Rechteck auf der Kugeloberfläche (zwei Dreiecke)

      MeshGeometry3D mesh = new MeshGeometry3D();

      mesh.Positions.Add(pt0);
      mesh.Positions.Add(pt1);
      mesh.Positions.Add(pt2);
      mesh.Positions.Add(pt3);

      mesh.TriangleIndices.Add(0);
      mesh.TriangleIndices.Add(1);
      mesh.TriangleIndices.Add(2);

      Vector3D norm = CalcNormal(pt0, pt1, pt2);

      mesh.Normals.Add(norm);
      mesh.Normals.Add(norm);
      mesh.Normals.Add(norm);

      mesh.TriangleIndices.Add(0);
      mesh.TriangleIndices.Add(2);
      mesh.TriangleIndices.Add(3);

      mesh.Normals.Add(norm);
      mesh.Normals.Add(norm);
      mesh.Normals.Add(norm);

      // Material zusammenfügen
      Material mat = new DiffuseMaterial(new SolidColorBrush(Colors.Blue));
      GeometryModel3D model = new GeometryModel3D(mesh, mat);
      Model3DGroup group = new Model3DGroup();
      group.Children.Add(model);

      return group;
    }

    private Vector3D CalcNormal(Point3D pt0, Point3D pt1, Point3D pt2)
    {
      // Vektor-Normale berechnen
      Vector3D v0 = new Vector3D(pt1.X - pt0.X, pt1.Y - pt0.Y, pt1.Z - pt0.Z);
      Vector3D v1 = new Vector3D(pt2.X - pt1.X, pt2.Y - pt1.Y, pt2.Z - pt1.Z);

      return Vector3D.CrossProduct(v0, v1);
    }
  }
}
