﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Media3D;

namespace Wuerfel
{
  public partial class MainWindow : Window
  {
    public MainWindow()
    {
      InitializeComponent();
    }

    private void OnStartExit(object sender, EventArgs e)
    {
      this.Close();
    }

    private void OnStartWuerfel(object sender, EventArgs e)
    {
      // Platzierung der Kamera
      PerspectiveCamera camera = new PerspectiveCamera();
      camera.FarPlaneDistance = 100;
      camera.NearPlaneDistance = 1;
      camera.LookDirection = new Vector3D(-10, -10, -7);
      camera.UpDirection = new Vector3D(0, 1, 0);
      camera.Position = new Point3D(10, 10, 10);
      camera.FieldOfView = 70;

      //OrthographicCamera camera = new OrthographicCamera();
      //camera.FarPlaneDistance = 100;
      //camera.NearPlaneDistance = 1;
      //camera.LookDirection = new Vector3D(-10, -10, -7);
      //camera.UpDirection = new Vector3D(0, 1, 0);
      //camera.Position = new Point3D(10, 10, 10);
      //camera.Width = 20;

      this.view.Camera = camera;

      // Lichteinstellung
      ModelVisual3D lightModel = new ModelVisual3D();
      Vector3D lightVector = new Vector3D(-20, -30, -10);
      DirectionalLight light = new DirectionalLight(Colors.White, lightVector);
      lightModel.Content = light;

      // Modell erzeugen
      Model3DGroup wuerfel = new Model3DGroup();

      // Eckpunkte des Würfels definieren
      Point3D p0 = new Point3D(0, 0, 0);
      Point3D p1 = new Point3D(5, 0, 0);
      Point3D p2 = new Point3D(5, 0, 5);
      Point3D p3 = new Point3D(0, 0, 5);
      Point3D p4 = new Point3D(0, 5, 0);
      Point3D p5 = new Point3D(5, 5, 0);
      Point3D p6 = new Point3D(5, 5, 5);
      Point3D p7 = new Point3D(0, 5, 5);

      // Vorderseite
      wuerfel.Children.Add(TriangleModel(p3, p2, p6));
      wuerfel.Children.Add(TriangleModel(p3, p6, p7));

      // Rechte Fläche
      wuerfel.Children.Add(TriangleModel(p2, p1, p5));
      wuerfel.Children.Add(TriangleModel(p2, p5, p6));

      // Rückseite
      wuerfel.Children.Add(TriangleModel(p1, p0, p4));
      wuerfel.Children.Add(TriangleModel(p1, p4, p5));

      // Linke Fläche
      wuerfel.Children.Add(TriangleModel(p0, p3, p7));
      wuerfel.Children.Add(TriangleModel(p0, p7, p4));

      // Obere Fläche
      wuerfel.Children.Add(TriangleModel(p7, p6, p5));
      wuerfel.Children.Add(TriangleModel(p7, p5, p4));

      // Untere Fläche
      wuerfel.Children.Add(TriangleModel(p2, p3, p0));
      wuerfel.Children.Add(TriangleModel(p2, p0, p1));

      if (this.view.Children.Count == 0)
      {
        ModelVisual3D model = new ModelVisual3D();
        model.Content = wuerfel;
        this.view.Children.Add(lightModel);
        this.view.Children.Add(model);
      }
    }

    private Model3DGroup TriangleModel(Point3D pt0, Point3D pt1, Point3D pt2)
    {
      // Definition eines Dreiecks im Drahtmodell
      MeshGeometry3D mesh = new MeshGeometry3D();

      mesh.Positions.Add(pt0);
      mesh.Positions.Add(pt1);
      mesh.Positions.Add(pt2);

      mesh.TriangleIndices.Add(0);
      mesh.TriangleIndices.Add(1);
      mesh.TriangleIndices.Add(2);

      Vector3D norm = CalcNormal(pt0, pt1, pt2);

      mesh.Normals.Add(norm);
      mesh.Normals.Add(norm);
      mesh.Normals.Add(norm);

      Material mat = new DiffuseMaterial(new SolidColorBrush(Colors.Blue));
      GeometryModel3D model = new GeometryModel3D(mesh, mat);
      Model3DGroup group = new Model3DGroup();
      group.Children.Add(model);

      return group;
    }

    private Vector3D CalcNormal(Point3D pt0, Point3D pt1, Point3D pt2)
    {
      // Berechnung des Vektorproduktes
      Vector3D v0 = new Vector3D(pt1.X - pt0.X, pt1.Y - pt0.Y, pt1.Z - pt0.Z);
      Vector3D v1 = new Vector3D(pt2.X - pt1.X, pt2.Y - pt1.Y, pt2.Z - pt1.Z);

      return Vector3D.CrossProduct(v0, v1);
    }
  }
}
