﻿using System;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Data;
using System.Windows.Shapes;
using System.Windows.Media;

namespace DragAndDrop
{
  public partial class MainWindow : Window
  {
    private Point ptStart;
    private bool bIsDraging = false;

    public MainWindow()
    {
      InitializeComponent();
    }

    private void listBox_PreviewMouseLeftButtonDown(object sender, MouseButtonEventArgs e)
    {
      ptStart = e.GetPosition(null);
    }

    private void listBox_PreviewMouseMove(object sender, MouseEventArgs e)
    {
      if ((e.LeftButton == MouseButtonState.Pressed) && !bIsDraging)
      {
        Point ptCurrent = e.GetPosition(null);

        if (Math.Abs(ptCurrent.X - ptStart.X) > SystemParameters.MinimumHorizontalDragDistance ||
            Math.Abs(ptCurrent.Y - ptStart.Y) > SystemParameters.MinimumVerticalDragDistance)
        {
          Farbe col = listBox.SelectedItem as Farbe;

          if (col != null)
          {
            bIsDraging = true;
            DataObject data = new DataObject(typeof(Farbe), col);
            DragDrop.DoDragDrop(listBox, data, DragDropEffects.Copy);
          }
        }
      }
    }

    private void Rectangle_DragEnter(object sender, DragEventArgs e)
    {
      listBox.Background = Brushes.LightGray;     
    }

    private void Rectangle_DragOver(object sender, DragEventArgs e)
    {
      if (e.Data.GetDataPresent(typeof(Farbe)))
      {
        e.Effects = DragDropEffects.Copy;
      }
      else
      {
        e.Effects = DragDropEffects.None;
      }

      e.Handled = true;
    }

    private void Rectangle_DragLeave(object sender, DragEventArgs e)
    {
      listBox.Background = Brushes.White;
    }

    private void Rectangle_Drop(object sender, DragEventArgs e)
    {
      Rectangle rect = e.Source as Rectangle;
      Farbe col = (Farbe)e.Data.GetData(typeof(Farbe));
      rect.Fill = new SolidColorBrush(col.RGBWert);
      listBox.Background = Brushes.White;
      bIsDraging = false;
    }
  }
}
