﻿using System;
using System.Windows;
using System.Windows.Controls;
using System.Collections.Generic;

namespace Rechner
{
  public partial class MainWindow : Window
  {
    // Wenn True, bei der nächsten Taste die aktuelle TextBox löschen
    private bool bClearInput;
    // Wenn True, bei der nächsten Taste die aktuelle TextBox auf 
    // den Stack schieben und dann löschen
    private bool bClearEnter;
    // Der Stack für die Zahlen
    private Stack<double> stack = new Stack<double>();

    public MainWindow()
    {
      InitializeComponent();
      ClearAll();   // Alles löschen
    }

    private void OnZahl(object sender, RoutedEventArgs e)
    {
      // Eine Zahlentaste wurde angeklickt
      string strName = (sender as Button).Name;
      string strZahl = strName.Substring(3, 1);

      if (bClearEnter)
      {
        Push();
        bClearEnter = false;
        textInput.Text = strZahl;
      }
      else
      {
        if (bClearInput)
        {
          textInput.Text = strZahl;
          bClearInput = false;
        }
        else
        {
          textInput.AppendText(strZahl);
        }
      }
    }

    private void OnPunkt(object sender, RoutedEventArgs e)
    {
      // Die Punkt-Taste wurde angeklickt
      textInput.AppendText(".");
    }

    private void OnOperator(object sender, RoutedEventArgs e)
    {
      // Die Grundrechenarten
      if (stack.Count <= 0)
        return;

      double dZahl1 = 0.0;
      double dResult = 0.0;
      string strName = (sender as Button).Name;

      // Zahl aus der aktuellen TextBox
      double.TryParse(textInput.Text, out dZahl1);

      // Zahl aus dem Stack
      double dZahl2 = stack.Pop();

      switch (strName)
      {
        case "btnAdd":
          dResult = dZahl2 + dZahl1;
          break;
        case "btnSub":
          dResult = dZahl2 - dZahl1;
          break;
        case "btnMul":
          dResult = dZahl2 * dZahl1;
          break;
        case "btnDiv":
          dResult = dZahl2 / dZahl1;
          break;
      }

      textInput.Text = dResult.ToString();
      bClearEnter = true;
      ShowStack();
    }

    private void OnEnter(object sender, RoutedEventArgs e)
    {
      // Enter-Taste wurde gedrückt
      Push();
      bClearInput = true;
    }

    private void Push()
    {
      // Die Zahl aus der aktuellen TextBox auf den Stack schieben
      double dZahl = 0.0;

      double.TryParse(textInput.Text, out dZahl);
      stack.Push(dZahl);
      ShowStack();
    }

    private void ShowStack()
    {
      // Stack als Text ausgeben
      string strOut = string.Empty;

      foreach (double d in stack)
      {
        if (strOut == string.Empty)
        {
          strOut = d.ToString();
        }
        else
        {
          strOut = d.ToString() + "\r\n" + strOut;
        }
      }

      textStack.Text = strOut;
      textStack.ScrollToEnd();
    }

    private void OnSign(object sender, RoutedEventArgs e)
    {
      // Taste +/- wurde gedrückt
      bool bMinus = (textInput.Text.Substring(0, 1) == "-");

      if (bMinus)
      {
        textInput.Text = textInput.Text.Substring(1);
      }
      else
      {
        textInput.Text = "-" + textInput.Text;
      }
    }

    private void OnClear(object sender, RoutedEventArgs e)
    {
      // Taste CLR wurde gedrückt
      ClearAll();
    }

    private void ClearAll()
    {
      // Alles löschen bzw. zurücksetzen
      textInput.Text = "0";
      bClearInput = true;
      bClearEnter = false;
      stack.Clear();
      textStack.Clear();
    }
  }
}
