#include "stdafx.h"
#include "Listing.h"
#include "ChildView.h"

#pragma managed 

using namespace System;
using namespace System::Windows;
using namespace System::Windows::Controls;
using namespace System::Windows::Interop;
using namespace System::Windows::Media;
using namespace System::Windows::Media::Imaging;
using namespace System::Windows::Media::Animation;
using namespace System::Windows::Media::Media3D;

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

int iRinge;
int iSeg;
bool bInit;

void Init3D(CWnd* pWnd);
HwndSource^ Create3D(HWND hwndParent, int x, int y, int width, int height) ;
Vector3D CalcNormal(Point3D pt0, Point3D pt1, Point3D pt2);
Model3DGroup^ TriangleModel(Point3D pt0, Point3D pt1, Point3D pt2, Point3D pt3, int iS, int iR);
Model3DGroup^ ModelKugel(int iRinge, int iSeg, double dSize);
void Define3DView(Viewport3D^ view);

// CChildView
#pragma unmanaged 

CChildView::CChildView()
{
}

CChildView::~CChildView()
{
}

BEGIN_MESSAGE_MAP(CChildView, CWnd)
	ON_WM_PAINT()
END_MESSAGE_MAP()

// CChildView-Meldungshandler

BOOL CChildView::PreCreateWindow(CREATESTRUCT& cs) 
{
	if (!CWnd::PreCreateWindow(cs))
		return FALSE;

	cs.dwExStyle |= WS_EX_CLIENTEDGE;
	cs.style &= ~WS_BORDER;
	cs.lpszClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, 
		::LoadCursor(NULL, IDC_ARROW), reinterpret_cast<HBRUSH>(COLOR_WINDOW+1), NULL);

	return TRUE;
}

void CChildView::OnPaint() 
{
	CPaintDC dc(this); // Gertekontext zum Zeichnen
	
  if(!bInit)
	  Init3D(this);

}
#pragma managed 

HwndSource^ Create3D(HWND hwndParent, int x, int y, int width, int height) 
{
  Viewport3D^ view = gcnew Viewport3D();
  
  HwndSourceParameters params;
  
  params.ParentWindow = IntPtr(hwndParent);
  params.WindowStyle = WS_CHILD | WS_VISIBLE;
  params.PositionX = x;
  params.PositionY = y;
  params.Width = width;
  params.Height = height;
  HwndSource^ src = gcnew HwndSource(params);
  
  src->RootVisual = view;
  
  return src;
}

void Init3D(CWnd* pWnd) 
{
  iRinge = 32;
  iSeg = 32;

  HWND hwndParent = pWnd->GetSafeHwnd();
  RECT rect = { 0 };

  GetWindowRect(hwndParent, &rect);
  MapWindowPoints(0, hwndParent, (POINT*)&rect, 2);

  // WPF-Fenster
  HwndSource^ srcWpfView = Create3D(hwndParent, rect.left, rect.top, 
                                    rect.right - rect.left, rect.bottom - rect.top);

  Viewport3D^ view = (Viewport3D^)srcWpfView->RootVisual;
  Define3DView(view);
  ShowWindow((HWND)srcWpfView->Handle.ToPointer(), SW_SHOW);

  bInit = true;
}

// Das ist der Algorithmus fr die 3D-Darstellung einer Kugel
// mit dem Weltbild

void Define3DView(Viewport3D^ view)
{
  AxisAngleRotation3D^ ax;

  // Kamera definieren
  PerspectiveCamera^ camera = gcnew PerspectiveCamera();
  camera->FarPlaneDistance = 100;
  camera->NearPlaneDistance = 1;
  camera->LookDirection = Vector3D(-10, -10, -10);
  camera->UpDirection = Vector3D(0, 1, 0);
  camera->Position = Point3D(10, 10, 10);
  camera->FieldOfView = 70;
  view->Camera = camera;

  // Licht definieren
  ModelVisual3D^ lightModel = gcnew ModelVisual3D();
  Vector3D lightVector = Vector3D(-20, -30, -10);
  DirectionalLight^ light = gcnew DirectionalLight(Colors::White, lightVector);
  lightModel->Content = light;

  // Transformationen definieren
  ax = gcnew AxisAngleRotation3D(Vector3D(0, 1, 0), -150.0);
  RotateTransform3D^ rt = gcnew RotateTransform3D(ax, Point3D(0, 0, 0));

  double dRadius = 5.0;

  if (view->Children->Count == 0)
  {
    ModelVisual3D^ model = gcnew ModelVisual3D();
    model->Content = ModelKugel(iRinge, iSeg, dRadius);
    model->Transform = rt;
    view->Children->Add(lightModel);
    view->Children->Add(model);

    DoubleAnimation^ anim = gcnew DoubleAnimation();
    anim->Duration = Duration(TimeSpan::FromSeconds(20));
    anim->From = 0;
    anim->To = 360;
    anim->RepeatBehavior = RepeatBehavior::Forever;

    ax->BeginAnimation(AxisAngleRotation3D::AngleProperty, anim);
  }
}

Vector3D CalcNormal(Point3D pt0, Point3D pt1, Point3D pt2)
{
  // Vektor-Normale berechnen
  Vector3D v0 = Vector3D(pt1.X - pt0.X, pt1.Y - pt0.Y, pt1.Z - pt0.Z);
  Vector3D v1 = Vector3D(pt2.X - pt1.X, pt2.Y - pt1.Y, pt2.Z - pt1.Z);

  return Vector3D::CrossProduct(v0, v1);
}

Model3DGroup^ TriangleModel(Point3D pt0, Point3D pt1, Point3D pt2, Point3D pt3, int iS, int iR)
{
  // Ein Rechteck auf der Kugeloberflche (zwei Dreiecke)
  MeshGeometry3D^ mesh = gcnew MeshGeometry3D();

  mesh->Positions->Add(pt0);
  mesh->Positions->Add(pt1);
  mesh->Positions->Add(pt2);
  mesh->Positions->Add(pt3);

  mesh->TriangleIndices->Add(0);
  mesh->TriangleIndices->Add(1);
  mesh->TriangleIndices->Add(2);

  Vector3D norm = CalcNormal(pt0, pt1, pt2);

  mesh->Normals->Add(norm);
  mesh->Normals->Add(norm);
  mesh->Normals->Add(norm);

  mesh->TriangleIndices->Add(0);
  mesh->TriangleIndices->Add(2);
  mesh->TriangleIndices->Add(3);

  mesh->Normals->Add(norm);
  mesh->Normals->Add(norm);
  mesh->Normals->Add(norm);

  // Verbindungspunkte zum Image definieren
  PointCollection^ pc = gcnew PointCollection();
  pc->Add(Point((double)iS / (double)iSeg, (double)iR / (double)iRinge));
  pc->Add(Point((double)iS / (double)iSeg, (double)(iR + 1) / (double)iRinge));
  pc->Add(Point((double)(iS + 1) / (double)iSeg, (double)(iR + 1) / (double)iRinge));
  pc->Add(Point((double)(iS + 1) / (double)iSeg, (double)iR / (double)iRinge));
  mesh->TextureCoordinates = pc;

  // ImageBrush als Material definieren und Modell mit dem
  // Material zusammenfgen
  BitmapImage^ img = gcnew BitmapImage(gcnew Uri("Welt.png", UriKind::RelativeOrAbsolute));
  ImageBrush^ brush = gcnew ImageBrush(img);
  brush->ViewportUnits = BrushMappingMode::Absolute;

  Material^ mat = gcnew DiffuseMaterial(brush);
  GeometryModel3D^ model = gcnew GeometryModel3D(mesh, mat);
  Model3DGroup^ group = gcnew Model3DGroup();
  group->Children->Add(model);

  return group;
}

Model3DGroup^ ModelKugel(int iRinge, int iSeg, double dSize)
{
  // Berechnung der Kugeloberflche in Segmenten und Ringen
  Model3DGroup^ kugel = gcnew Model3DGroup();

  double dRingWinkel = Math::PI / iRinge;
  double dSegWinkel = 2.0 * Math::PI / iSeg;

  for (int iR = 0; iR < iRinge; iR++)
  {
    double r0 = Math::Sin(iR * dRingWinkel);
    double r1 = Math::Sin((iR + 1) * dRingWinkel);
    double y0 = Math::Cos(iR * dRingWinkel);
    double y1 = Math::Cos((iR + 1) * dRingWinkel);

    for (int iS = 0; iS <= iSeg; iS++)
    {
      double x0 = r0 * Math::Sin(iS * dSegWinkel);
      double z0 = r0 * Math::Cos(iS * dSegWinkel);
      double x1 = r1 * Math::Sin(iS * dSegWinkel);
      double z1 = r1 * Math::Cos(iS * dSegWinkel);

      Point3D pt1 = Point3D(x0 * dSize, y0 * dSize, z0 * dSize);
      Point3D pt2 = Point3D(x1 * dSize, y1 * dSize, z1 * dSize);

      x0 = r0 * Math::Sin((iS + 1) * dSegWinkel);
      z0 = r0 * Math::Cos((iS + 1) * dSegWinkel);
      x1 = r1 * Math::Sin((iS + 1) * dSegWinkel);
      z1 = r1 * Math::Cos((iS + 1) * dSegWinkel);

      Point3D pt3 = Point3D(x0 * dSize, y0 * dSize, z0 * dSize);
      Point3D pt4 = Point3D(x1 * dSize, y1 * dSize, z1 * dSize);

      kugel->Children->Add(TriangleModel(pt1, pt2, pt4, pt3, iS, iR));
    }
  }

  return kugel;
}

